
from catalog import ImageCatalog, Sequence
# from imageengine import Toto
from imageengine import IImageEngine
from imageengine import PixelType
from preprocessing import WhiteEstimator, correct_non_uniform_lighting

from template_matcher import TemplateMatcher

class TrapsDetector(object):

    def __init__(self, template_matcher):
        """
        :param TemplateMatcher template_matcher:
        """
        self.template_matcher = template_matcher

    def compute_traps_mask(self, sequence, channel_id, trap_aabb):
        """Remove the traps in the input sequence

        :param Sequence sequence:
        :param str channel_id:
        :param Aabb trap_aabb:
        :return Image: the mask of traps (binary image containing 0)

        the idea of this method is as follows:
        1. allow the user to provide an axis-aligned bounding box that contains a trap, and use this area as a pattern to be searched in all images of the sequence. Provided the tolerance is big enough, this should find all traps in the sequence.
        2. compute a clean (without globules) trap, by computing for each pixel, the median value from all traps for that pixel.
        3. For each trap location, remove the trap by substracting the clean trap. This should leave the globules only.

        At the end of this image processing, we expect to only have the globules in the image.
        """
        uniform_stack = correct_non_uniform_lighting(sequence, channel_id, white_estimator=WhiteEstimator(open_size=75, close_size=75, average_size=75))

        first_image = uniform_stack.get_image(frame_index=0)
        trap_image = first_image.get_subimage(trap_aabb)
        # ie.save_as_tiff(trap_image, '/home/graffy/Desktop/template.tiff')

        matches = self.template_matcher.match_template(first_image, trap_image)
        
        #non_uniform_stack = sequence.as_stack()
        #uniform_stack = IImageEngine.get_instance().divide(non_uniform_stack, white_estimate)
        # IImageEngine.get_instance().save_as_tiff(white_estimate, './white_estimate.tiff')
        return matches
        